%1D Optimization; Olivier Schwab 10.10.2013, edited by Philipp Hahn 2014

%close previous sessions of parallel workers (if there are any)
matlabpool close force local

%clear everything
clc
clear all;
close all;

%%Settings: CHANGE THESE BEFORE EXECUTION
%number of processors
c=4;
%comsol installation folder
comsol_path='C:\APPS\Comsol43a';
%change current directory
%cd('K:\Documents\Publications\Optimization\new_files_1D')
%temporary directory
temp_path=[cd '\Temp'];
% load old population? yes->1 no->0
oldpop=0;
%ranges are set in function set_ranges !!
%normalization ranges are set in function set_normalization !!
%material properties are set in the objective foptim20 !!
%%__________________________________________________________________________________________________


%start parallel workers and comsol server
start_workers(c,comsol_path,temp_path);

%create appropriate number of folders and copy everything in there; return
%time of execution
setup_date=file_setup_init(c);

%set ranges
[freq_l,freq_u,w_carrier_l,w_carrier_u,w_fluid_l,w_fluid_u,w_reflector_l,w_reflector_u]=set_ranges;

%normalize parameter ranges
[scale_factor_freq,scale_factor_w_carrier,scale_factor_w_fluid,scale_factor_w_reflector,freq_l_n,freq_u_n,w_carrier_l_n,w_carrier_u_n,w_fluid_l_n,w_fluid_u_n,w_reflector_l_n,w_reflector_u_n]=set_normalization(freq_l,freq_u,w_carrier_l,w_carrier_u,w_fluid_l,w_fluid_u,w_reflector_l,w_reflector_u);
scale_factor=[scale_factor_freq,scale_factor_w_carrier,scale_factor_w_fluid,scale_factor_w_reflector];
range_l=[freq_l_n,w_carrier_l_n,w_fluid_l_n,w_reflector_l_n];
range_u=[freq_u_n,w_carrier_u_n,w_fluid_u_n,w_reflector_u_n];

%start timer
tic;

%load options and execute genetic algorithm
%gaproblem=set_ga_options(scale_factor,range_n,ga_para,setup_date);
if oldpop
    pp=load([cd '\latestpop.mat']);
    init_old=pp.pop;
    options=gaoptimset('InitialPopulation', init_old,'PopulationSize',100,'Generations',400,'CrossoverFraction',0.5,'InitialPenalty',1000,'PenaltyFactor',1000,'Display','iter','PlotFcns',@gaplotbestf,'OutputFcns',@(options,state,flag)savebestindividual(options,state,flag,scale_factor,setup_date),'UseParallel','always','TolC',1e-20,'TolF',1e-20);
else
    options=gaoptimset('PopulationSize',100,'Generations',200,'CrossoverFraction',0.5,'InitialPenalty',1000,'PenaltyFactor',1000,'Display','iter','PlotFcns',@gaplotbestf,'OutputFcns',@(options,state,flag)savebestindividual(options,state,flag,scale_factor,setup_date),'UseParallel','always','TolC',1e-20,'TolF',1e-20);
end
[para,fval]=ga(@(para)foptim14(para,scale_factor,setup_date),4,[],[],[],[],range_l,range_u,[],options);

%end time for GA
t=toc;

%here a second algorithm could be called with initial values the genetic
%alg. results; eg pattern search

%denormalize results
npara=de_normalize(para,scale_factor);

if npara(2)<50e-6
    npara(2)=0;
end

%display results
disp(' ');
disp('Results from GA:');
disp(['f_best = ',num2str(npara(1)),'Hz']);
disp(['w_carrier = ',num2str(npara(2)),'m']);
disp(['w_fluid = ',num2str(npara(3)),'m']);
disp(['w_reflector = ',num2str(npara(4)),'m']);
disp(['Computing time: ',num2str(t),'s']);
disp(['Fitness value: ',num2str(fval)]);

%remove temp folder
temporary_directory=[cd '\Temp'];
if exist(temporary_directory)==7
   system(['rmdir /s /q ' '"' temporary_directory '"']); 
end

%close multiple workers
matlabpool close;