function [state, options,optchanged] = savebestindividual(options,state,flag,scale_factor,setup_date)
%GAOUTPUTFCNTEMPLATE Template to write custom OutputFcn for GA.
%   [STATE, OPTIONS, OPTCHANGED] = GAOUTPUTFCNTEMPLATE(OPTIONS,STATE,FLAG)
%   where OPTIONS is an options structure used by GA.
%
%   STATE: A structure containing the following information about the state
%   of the optimization:
%             Population: Population in the current generation
%                  Score: Scores of the current population
%             Generation: Current generation number
%              StartTime: Time when GA started
%               StopFlag: String containing the reason for stopping
%              Selection: Indices of individuals selected for elite,
%                         crossover and mutation
%            Expectation: Expectation for selection of individuals
%                   Best: Vector containing the best score in each generation
%        LastImprovement: Generation at which the last improvement in
%                         fitness value occurred
%    LastImprovementTime: Time at which last improvement occurred
%
%   FLAG: Current state in which OutputFcn is called. Possible values are:
%         init: initialization state
%         iter: iteration state
%    interrupt: intermediate state
%         done: final state
%
%   STATE: Structure containing information about the state of the
%          optimization.
%
%   OPTCHANGED: Boolean indicating if the options have changed.
%
%	See also PATTERNSEARCH, GA, GAOPTIMSET

%   Copyright 2004-2005 The MathWorks, Inc.
%   $Revision: 1.1 $  $Date: 2008/05/13 07:54:54 $

optchanged = false;

%para
a=2.5e-6;
rhoP=1050;
kappaP=.72*1/(1000*1500^2);
rho0=1000;
kappa0=1/(1000*1500^2);
c0=1500;

f1=1-kappaP/kappa0;
f2=2*(rhoP/rho0-1)/(2*rhoP/rho0+1);

% find best individual
bestScore = find(state.Score == min(state.Score),1);
para = state.Population(bestScore,:);
g = state.Generation;
pop=state.Population;

%save latest pop
save([cd '\latestpop.mat'],'pop')

%parameters
para_n=de_normalize(para,scale_factor);

if para_n(2)<50e-6
    para_n(2)=0;
end

switch flag
    case 'init'
        fid=fopen([cd '\results_' setup_date '.txt'],'a');
        fprintf(fid,'%s%f%s%i%s%6.12f%s%6.12f%s%6.12f%s%6.12f%s%e%s%e\n','Total time:',toc,' ; Initialization',0,': freq=',para_n(1),', w_carrier=',para_n(2),', w_fluid=',para_n(3),', w_reflector=',para_n(4),', f=',min(state.Score),', fmean=',mean(state.Score));
        fclose(fid);
    case {'iter','interrupt'}
        % save it
        fid=fopen([cd '\results_' setup_date '.txt'],'a');
        fprintf(fid,'%s%f%s%i%s%6.12f%s%6.12f%s%6.12f%s%6.12f%s%e%s%e\n','Total time:',toc,' ; Generation',g,': freq=',para_n(1),', w_carrier=',para_n(2),', w_fluid=',para_n(3),', w_reflector=',para_n(4),', f=',min(state.Score),', fmean=',mean(state.Score));
        fclose(fid);
        
    case 'done'
        % Something
end


