%3D Optimization; Olivier Schwab 7.10.2013 edited by Philipp Hahn 2014

%close previous sessions of parallel workers (if there are any)
matlabpool close force local

%clear everything
clc
clear all;
close all;

%%Settings: CHANGE THESE BEFORE EXECUTION
%number of processors
c=4;
%comsol installation folder
comsol_path='C:\APPS\Comsol43a';
%change current directory
%cd('K:\Documents\Publications\Optimization\Optimization_data_for_paper\3D\new_files3D_1')
%temporary directory
temp_path=[cd '\Temp'];
%load old population, yes->oldpop=1, no->oldpop=0; (for first calculation set to 0)
ga_para.oldpop=0;
%genetic algorithm settings: population size, generations, crossover, tolfun, tolcon
ga_para.Pop=200;
ga_para.Gen=200;
ga_para.Crossover=0.5;
ga_para.TolF=1e-25;
ga_para.TolC=1e-20;

%ranges are set in function set_ranges !!
%normalization ranges are set in function set_normalization !!
%material properties are set in the objective foptim20 !!
%%__________________________________________________________________________________________________


%start parallel workers and comsol server
start_workers(c,comsol_path,temp_path);

%create appropriate number of folders and copy everything in there; return
%time of execution
setup_date=file_setup_init(c);

%set ranges
[freq_l,freq_u,x_piezo_l,x_piezo_u,y_piezo_l,y_piezo_u,pos_x_l,pos_x_u,pos_y_l,pos_y_u,y_gl_l,y_gl_u]=set_ranges;

%normalize parameter ranges
[scale_factor,range_n_l,range_n_u]=set_normalization(freq_l,freq_u,x_piezo_l,x_piezo_u,y_piezo_l,y_piezo_u,pos_x_l,pos_x_u,pos_y_l,pos_y_u,y_gl_l,y_gl_u);

%start timer
tic;

%load options and execute genetic algorithm
%constraints:
Aineq=[0,   scale_factor(2),    0,                  scale_factor(4),    0,                  0; ...
       0,   0,                  scale_factor(3),    0,                  scale_factor(4),    -scale_factor(5)];
bineq=[4e-3;2.5e-3];

if ga_para.oldpop
    pp=load([cd '\latestpop.mat']);
    init_old=pp.pop;
    options=gaoptimset('InitialPopulation', init_old,'PopulationSize',ga_para.Pop,'Generations',ga_para.Gen,'CrossoverFraction',ga_para.Crossover,'Display','iter','InitialPenalty',1000,'PenaltyFactor',1000,'UseParallel','always','TolF',ga_para.TolF,'TolC',ga_para.TolC,'PlotFcns',@gaplotbestf,'OutputFcns',@(options,state,flag)savebestindividual(options,state,flag,scale_factor,setup_date));
else
    options=gaoptimset('PopulationSize',ga_para.Pop,'Generations',ga_para.Gen,'CrossoverFraction',ga_para.Crossover,'Display','iter','InitialPenalty',1000,'PenaltyFactor',1000,'UseParallel','always','TolF',ga_para.TolF,'TolC',ga_para.TolC,'PlotFcns',@gaplotbestf,'OutputFcns',@(options,state,flag)savebestindividual(options,state,flag,scale_factor,setup_date));
end


[para,fval]=ga(@(para)foptim20(para,scale_factor,setup_date),6,Aineq,bineq,[],[],range_n_l,range_n_u,[],options);

%end time for GA
t=toc;

%here a second algorithm could be called with initial values the genetic
%alg. results; eg pattern search

%display results
disp(' ');
disp('Results from GA:');
disp(['f_best = ',num2str(para(1)*scale_factor(1)),'Hz']);
disp(['x_piezo_best = ',num2str(para(2)*scale_factor(2)),'m']);
disp(['y_piezo_best = ',num2str(para(3)*scale_factor(3)),'m']);
disp(['pos_x = ',num2str(para(4)*scale_factor(4)),'m']);
disp(['pos_y = ',num2str(para(5)*scale_factor(5)),'m']);
disp(['l_square = ',num2str(para(6)*scale_factor(6)),'m']);
disp(['Computing time: ',num2str(t),'s']);
disp(['Fitness value: ',num2str(fval)]);

%remove temp folder
temporary_directory=[cd '\Temp'];
if exist(temporary_directory)==7
   system(['rmdir /s /q ' '"' temporary_directory '"']); 
end

%close multiple workers
matlabpool close;